import { create } from 'qrcode';
import { createReadStream } from 'fs';
import { make as make$1, encodePNGToStream, decodePNGFromStream } from 'pureimage';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = function(d, b) {
    extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
    return extendStatics(d, b);
};

function __extends(d, b) {
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function __generator(thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
}

/**
 * A helper class, which wraps all encoding/decoding of images in Node or the Browser.
 */
var CanvasLoader = /** @class */ (function () {
    function CanvasLoader(width, height) {
        this.canvas = this.makeCanvas(width, height);
    }
    CanvasLoader.prototype.toSVG = function () { throw Error('toSVG() is not implemented for this QR Code Type.'); };
    return CanvasLoader;
}());

var CanvasSVG = /** @class */ (function (_super) {
    __extends(CanvasSVG, _super);
    function CanvasSVG(width, height) {
        return _super.call(this, width, height) || this;
    }
    CanvasSVG.prototype.makeCanvas = function (width, height) {
        return new SVGCanvas(width, height);
    };
    CanvasSVG.prototype.loadImage = function (image) {
        // @ts-ignore
        return this.canvas.loadImage(image);
    };
    CanvasSVG.prototype.toBlob = function (mimeType, quality) {
        throw Error('Blob conversion is not implemented for SVG QR Codes.');
    };
    CanvasSVG.prototype.toStream = function (stream) {
        throw Error('toStream is not implemented for SVG QR Codes.');
    };
    CanvasSVG.prototype.toSVG = function () {
        // @ts-ignore
        return this.canvas.toSVG();
    };
    return CanvasSVG;
}(CanvasLoader));
var SVGCanvas = /** @class */ (function () {
    function SVGCanvas(width, height) {
        this.shapeCache = {};
        this.defs = {};
        this.shapes = [];
        this.col = 'white';
        this.width = width;
        this.height = height;
    }
    SVGCanvas.prototype.getContext = function () {
        return this;
    };
    Object.defineProperty(SVGCanvas.prototype, "fillStyle", {
        set: function (value) {
            this.col = value;
        },
        enumerable: false,
        configurable: true
    });
    SVGCanvas.prototype.fillRect = function (x, y, width, height) {
        var key = width + "-" + height + "-" + this.col;
        var newID = "s" + Object.values(this.defs).length;
        var rect = this.shapeCache[key];
        if (!rect) {
            this.defs[newID] = "<rect width=\"" + width + "\" height=\"" + height + "\" fill=\"" + this.col + "\" id=\"" + newID + "\"/>";
            this.shapeCache[key] = newID;
            rect = newID;
        }
        this.shapes.push({ rect: rect, x: x, y: y });
    };
    /**
     * Mocks a dummy image object so the existing render commands work.
     *
     * @param img
     * @return {{img: string, width: number, height: number}}
     */
    SVGCanvas.prototype.loadImage = function (img) {
        if (typeof img !== 'string') {
            throw Error('Invalid image input - SVG Mode only supports pure text background images!');
        }
        return {
            width: 1,
            height: 1,
            img: ("" + img).replace(/<[?\s]*(xml)/gm, function (m) { return m.replace('xml', 'misc'); })
        };
    };
    SVGCanvas.prototype.drawImage = function (imageObj, xx, yy, ow, oh, dx, dy, dw, dh) {
        var txt = "<svg x=\"" + dx + "\" y=\"" + dy + "\" width=\"" + dw + "\" height=\"" + dh + "\">" + imageObj.img + "</svg>";
        this.shapes.push({ txt: txt });
    };
    SVGCanvas.prototype.toSVG = function () {
        var out = "<svg xmlns=\"http://www.w3.org/2000/svg\" version=\"1.1\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" width=\"" + this.width + "\" height=\"" + this.height + "\">\n";
        out += '<defs>\n' + Object.values(this.defs).join('\n') + '</defs>\n';
        out += this.shapes.map(function (s) { return s.txt || "<use href=\"#" + s.rect + "\" x=\"" + s.x + "\" y=\"" + s.y + "\"/>"; }).join('\n');
        return out + "</svg>";
    };
    return SVGCanvas;
}());

var LoaderClass = null; // TODO: SVG Loader support.
/**
 * @hidden
 * @param newLoader
 */
var setLoaderClass = function (newLoader) {
    LoaderClass = newLoader;
};
/**
 * Generates a QR code with an image background.
 *
 * @param data The data to encode within the image.
 * @param backgroundPath A path to the background PNG to use. In browser, should be a URL.
 * @param props Customize the QR code using these properties.
 *
 * @see {@link https://www.npmjs.com/package/qrcode#manual-mode QRCode data structure}
 */
var make = function (data, backgroundPath, props) { return __awaiter(void 0, void 0, void 0, function () {
    var _a, qrOpts, colors, size, useSVG, qrCode, boxSize, boxScale, bits, qrWidth, reserved, w, h, loader, background, img, ctx, x, y;
    return __generator(this, function (_b) {
        switch (_b.label) {
            case 0:
                if (!LoaderClass)
                    throw Error("Loader was not set.");
                _a = props || {}, qrOpts = _a.qrOpts, colors = _a.colors, size = _a.size, useSVG = _a.useSVG;
                qrOpts = qrOpts || {};
                colors = Object.assign({
                    dark: 'rgba(0,0,0, 1.0)',
                    light: 'rgba(255,255,255, 0.75)',
                    overlay: ''
                }, colors || {});
                size = size || { boxSize: 6, scale: 0.35 };
                qrCode = create(data, qrOpts);
                if (!(qrCode.version === 1)) return [3 /*break*/, 2];
                return [4 /*yield*/, make(data, backgroundPath, __assign(__assign({}, props), { qrOpts: __assign(__assign({}, qrOpts), { version: 2 }) }))];
            case 1: 
            // V2 makes the image more robust by adding bottom right square, so require v2 minimum.
            return [2 /*return*/, _b.sent()];
            case 2:
                boxSize = size.boxSize || 6;
                boxScale = size.scale || 0.35;
                bits = new Uint8Array(qrCode.modules.data);
                qrWidth = qrCode.modules.size;
                reserved = new Uint8Array(qrCode.modules.reservedBit);
                w = (qrWidth + 2) * boxSize, h = (Math.ceil(bits.length / qrWidth) + 2) * boxSize;
                loader = useSVG ? new CanvasSVG(w, h) : new LoaderClass(w, h);
                return [4 /*yield*/, loader.loadImage(backgroundPath)];
            case 3:
                background = _b.sent();
                return [4 /*yield*/, loader.canvas];
            case 4:
                img = _b.sent();
                ctx = img.getContext('2d');
                ctx.fillStyle = colors.light;
                ctx.fillRect(0, 0, img.width, img.height);
                ctx.drawImage(background, 0, 0, background.width, background.height, // source dimensions
                boxSize, boxSize, img.width - boxSize * 2, img.height - boxSize * 2 // destination dimensions
                );
                if (colors.overlay) {
                    ctx.fillStyle = colors.overlay;
                    ctx.fillRect(0, 0, img.width, img.height);
                }
                x = 0, y = 0;
                bits.forEach(function (b, idx) {
                    if (idx && 0 === (idx % qrWidth)) {
                        y += 1;
                        x = 0;
                    }
                    var important = reserved[idx];
                    var ox = (x + 1) * boxSize, oy = (y + 1) * boxSize, square = boxSize;
                    if (!important) {
                        square = Math.ceil(boxSize * boxScale);
                        ox += Math.ceil((boxSize - square) / 2);
                        oy += Math.floor((boxSize - square) / 2);
                    }
                    // @ts-ignore
                    ctx.fillStyle = b ? colors.dark : colors.light;
                    ctx.fillRect(ox, oy, square, square);
                    x += 1;
                });
                return [2 /*return*/, loader];
        }
    });
}); };

/**
 * Node Canvas implementation.
 */
var NodeLoader = /** @class */ (function (_super) {
    __extends(NodeLoader, _super);
    function NodeLoader() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    NodeLoader.prototype.loadImage = function (image) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, decodePNGFromStream(createReadStream(image))];
            });
        });
    };
    NodeLoader.prototype.makeCanvas = function (width, height) {
        return make$1(width, height);
    };
    NodeLoader.prototype.toBlob = function (_mimeType, _quality) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                throw Error('Cannot encode canvas to Blob inside Node! Use "toStream" instead.');
            });
        });
    };
    NodeLoader.prototype.toStream = function (stream) {
        return encodePNGToStream(this.canvas, stream);
    };
    return NodeLoader;
}(CanvasLoader));

setLoaderClass(NodeLoader);

export default make;
//# sourceMappingURL=index.es.js.map
