import * as QRCode from 'qrcode';
import { QRCodeOptions } from "qrcode";
import CanvasLoader from "./canvas/canvas-wrapper";
/**
 * @hidden
 * @param newLoader
 */
export declare const setLoaderClass: (newLoader: any) => void;
export interface QRStyleOpts {
    /**
     * The Version/error level to use. See [node-qrcode](https://www.npmjs.com/package/qrcode#error-correction-level).
     */
    qrOpts?: QRCodeOptions;
    /**
     * The individual colors used in the QR code.
     * All colors can be specified in rgba(), or any other HTML string.
     *
     * If `overlay` is set to a non-empty string, covers the image with this color to increase contrast. Use rgba().
     */
    colors?: {
        dark: string;
        light: string;
        overlay: string;
    };
    /**
     * Adjust the scaling of the image.
     */
    size?: {
        /**
         * The size per-square in pixels.
         */
        boxSize: number;
        /**
         * The scale to resize data squares.
         */
        scale: number;
    };
    useSVG?: boolean;
}
/**
 * Generates a QR code with an image background.
 *
 * @param data The data to encode within the image.
 * @param backgroundPath A path to the background PNG to use. In browser, should be a URL.
 * @param props Customize the QR code using these properties.
 *
 * @see {@link https://www.npmjs.com/package/qrcode#manual-mode QRCode data structure}
 */
export declare const make: (data: string | QRCode.QRCodeSegment[], backgroundPath: string, props?: QRStyleOpts | undefined) => Promise<CanvasLoader>;
